package functionaljavatest;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.util.ArrayList;
import javax.swing.JPanel;

/**
 *
 * @author ubik
 */
class GameCanvas extends JPanel {
    
    private final int[] COLORS = new int[]{0xff1417,0xff6611,0xff8844,0xffee55,
                                           0xfefe38,0xffff99,0xaacc22,0xbbdd77,
                                           0xc8cf82,0x92a77e,0x5599ee,0x0088cc,
                                           0x226688,0x175279,0x557777,0xddbb33,
                                           0xd3a76d,0xa9834b,0xaa6688,0x767676  };

    private Game game;
    private BufferedImage buffer;
    private static final int WIDTH = 450;
    private static final int HEIGHT = 450;
    private static final int OFFSET = 10;

    public GameCanvas() {
        game = new Game(this);

        // Gravity
        // game.addEquation(Physics.gravity(new Vec(0.1, 0.1)));

        game.addEquation(Physics.max(4.0));        
        
        game.addEquation(Physics.bounce(WIDTH, HEIGHT, 0.9));
                
        game.addEquation(Physics.attr(Pnt.random(WIDTH,HEIGHT), Math.random()*5));
        game.addEquation(Physics.attr(Pnt.random(WIDTH,HEIGHT), Math.random()*5));
        game.addEquation(Physics.attr(Pnt.random(WIDTH,HEIGHT), Math.random()*5));
        
        setOpaque(false);
        createBuffer();
        
        addMouseListener(new MouseAdapter() {

            @Override
            public void mousePressed(MouseEvent me) {
                game.addSprite(new Pnt(me.getPoint()));
            }
        });
    }

    private void createBuffer() {
        buffer = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
        Graphics graphics = buffer.getGraphics();

        graphics.setColor(Color.BLACK);
        graphics.fillRect(0, 0, WIDTH, HEIGHT);
    }

    public void redraw() {
        // revalidate();
        repaint();
    }

    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.setColor(Color.BLACK);
        g.fillRect(0, 0, getWidth(), getHeight());

        Graphics2D graphics = (Graphics2D) buffer.getGraphics();
        graphics.setColor(Color.BLACK);
        // graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,0.1f));        
        // graphics.fill(new Rectangle(0, 0, WIDTH, HEIGHT));
        
        final float[] matrix = {
            0.111f, 0.111f, 0.111f,
            0.111f, 0.111f, 0.111f,
            0.111f, 0.111f, 0.111f,};

        BufferedImageOp op = new ConvolveOp(new Kernel(3, 3, matrix));
        buffer = op.filter(buffer, null);        
        graphics = (Graphics2D) buffer.getGraphics();
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);        

        final int size = 5;
        graphics.setColor(Color.WHITE);

        ArrayList<Pnt> positions = game.getPositions();
        Ellipse2D.Double s = new Ellipse2D.Double(0, 0, size, size);
        int count = 0;
        for (Pnt p : positions) {
            graphics.setColor(new Color(COLORS[count % COLORS.length]));
            s.x = p.x;
            s.y = p.y;
            graphics.fill(s);
            
            count++;
        }

        // graphics.drawRect(0, 0, WIDTH-1, HEIGHT-1);

        g.drawImage(buffer, OFFSET, OFFSET, this);
    }
}
