/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package functionaljavatest;

import fj.F2;

/**
 *
 * @author ubik
 */
public class Physics {

    public static F2<Pnt, Vec, Vec> gravity(final Vec dir) {
        return new F2<Pnt, Vec, Vec>() {
            @Override
            public Vec f(Pnt a, Vec b) {
                return b.add(dir.x, dir.y);
            }
        };
    }

    public static F2<Pnt, Vec, Vec> bounce(final double width, final double height, final double bounce) {
        return new F2<Pnt, Vec, Vec>() {

            @Override
            public Vec f(Pnt pos, Vec vel) {
                Vec res = vel.copy();
                if ((vel.x > 0 && pos.x > width )
                        || vel.x < 0 && pos.x < 0) {
                    res.x = (-res.x)*bounce;
                }
                if ((vel.y > 0 && pos.y > height )
                        || vel.y < 0 && pos.y < 0) {
                    res.y = (-res.y)*bounce;
                }

                return res;
            }
        };
    }
    
    public static F2<Pnt, Vec, Vec> whirl( final Pnt center, final double strength ) {
        return new F2<Pnt, Vec, Vec>() {

            @Override
            public Vec f(Pnt pos, Vec vel) {
                Vec diff = center.diff(pos);
                double len = diff.length();
                
                if( len <= 0.0001 )
                    return vel.copy();
                
                return vel.add( diff.normal().scale( strength/(len)) );
            }
        };        
    }
    
    public static F2<Pnt, Vec, Vec> attr( final Pnt cntr, final double strength ) {
        final Pnt center = cntr.copy();
        return new F2<Pnt, Vec, Vec>() {
            @Override
            public Vec f(Pnt pos, Vec vel) {
                Vec diff = center.diff(pos);
                double len = diff.length();
                
                if( len <= 0.0001 )
                    return vel.copy();
                
                return vel.add( diff.scale( strength/(len*len)) );
            }
        };        
    }    
    
    public static F2<Pnt, Vec, Vec> max( final double maxVelocity ) {
        return new F2<Pnt, Vec, Vec>() {

            @Override
            public Vec f(Pnt pos, Vec vel) {
                double len = vel.length();
                
                if( len <= maxVelocity )
                    return vel.copy();
                
                return vel.scale(maxVelocity/len);
            }
        };        
    }     
}
